/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * CharCell.h: Header file for character cell manipulation functions
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef CHARCELL_H
#define CHARCELL_H

#include <iostream.h>

/**
  A @var{CharCell} represents a character with attributes on a display. The
  character is represented with 16 bits, the forground and background
  colours are from a set of eight colours, and there is a flash bit.
 */

class CharCell {
  /// Character code of glyph.
  unsigned int glyph:16;
  /// Foreground colour.
  unsigned int fore:3;
  /// Background colour.
  unsigned int back:3;
  /// Flash attribute.
  BOOL flash:1;
  ///
  void clear(void) {glyph=32; fore=7; back=0; flash=FALSE;};
  ///
  void free(void) {clear();};

  ///
public:
  /// Default constructor.
  CharCell(void) {clear();};
  /// Copy constructor.
  CharCell(const CharCell &c) {*this=c;};
  /// Destructor.
  ~CharCell() {free();};

  /// Assignment operator.
  CharCell &operator=(const CharCell &c) {
    glyph=c.glyph; fore=c.fore; back=c.back; flash=c.flash;
    return *this;
  };

  /// @return nonzero if the @var{CharCell}s @var{c1} and @var{c2} are identical.
  friend BOOL operator==(const CharCell &c1, const CharCell &c2) {
    return (c1.glyph==c2.glyph)&&(c1.fore==c2.fore)&&(c1.back==c2.back)&&(c1.flash==c2.flash);
  }

  /// @return nonzero if the @var{CharCell}s @var{c1} and @var{c2} differ.
  friend BOOL operator!=(const CharCell &c1, const CharCell &c2) {
    return (c1.glyph!=c2.glyph)||(c1.fore!=c2.fore)||(c1.back!=c2.back)||(c1.flash!=c2.flash);
  };

  /// @return glyph character code.
  int getGlyph(void) { return glyph; };
  /// Sets the glyph character code to @var{n}.
  void setGlyph(int n) { glyph=n; };
  /// @return the foreground colour.
  int getFore(void) { return fore; };
  /// Sets the foreground colour to @var{n}.
  void setFore(int n) { fore=n; };
  /// @return the background colour.
  int getBack(void) { return back; };
  /// Sets the background colour to @var{n}.
  void setBack(int n) { back=n; };
  /// @return flashing attribute.
  BOOL getFlash(void) { return flash; };
  /// Sets the flashing attribute to @var{n}.
  void setFlash(BOOL n) { flash=n; };
  
  /// Show contents of the object.
  friend ostream &operator<<(ostream &s, const CharCell &c) {
    s << "(CharCell: " << c.glyph << ", " << c.fore << ", " << c.back << ", " << c.flash << ")";
    return s;
  }
};

#endif /* CHARCELL_H */
