/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * ConnectionDriver.h - prototypes for generic Connection driver class
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef CONNECTIONDRIVER_H
#define CONNECTIONDRIVER_H

#include <String.h>

class ConnectionDriver;

struct ConnectionDrivers {
  char *name;
  ConnectionDriver *(*newobject)(String &);
};

extern ConnectionDrivers ConnectionDrivers[];

/**

  @var{ConnectionDriver}s are the superclass of classes that provide
  generic stream access. Most of the functionality is provided by
  subclasses which perform, for example TCP/IP or serial access.

  */

class ConnectionDriver {
  ///
  void clear(void);
  ///
  void free(void);
  ///
public:
  /// Default constructor.
  ConnectionDriver(void);
  /// Destructor.
  virtual ~ConnectionDriver();

  /**

    This creates a new object of the type requested, which is implemented
    by a subclass of type @var{ConnectionDriver}. The connection to the
    server will have already been made on return. This method will fail if
    there the @var{type} is unknown, or the service cannot be contacted.
    Currently there is no indication of why it failed---this is a bug.
    
    @param type Type of @var{ConnectionDriver} (@code{"serial"}, @code{"tcp"})
    @param path Description of service, (@code{"localhost:23"})
    @return Pointer to a new object of the type requested, or NULL on failure.

    */
  friend ConnectionDriver *newConnectionDriver(const String &type, const String &path);

  /**

    Writes data to the @var{ConnectionDriver}.

    @param buf pointer to the buffer to write from.
    @param len length of buffer.
    @return number of bytes written, <=0 on failure.
    */
  virtual int write(const char *buf, int len)=0;
  
  /** Reads data from the @var{ConnectionDriver}.

    @param buf pointer to buffer to recieve data.
    @param len size of buffer.
    @return number of bytes actually read, <=0 on failure.
    */
  virtual int read(char *buf, int len)=0;

  /**

    In order to avoid blocking or busy waiting, this method is provided to
    give application a file descriptor to give to @code{select()} to
    monitor. It is possible that the driver doesn't use file descriptors,
    and you will get a dummy value instead which will cause @code{select()}
    to act as required by the driver. You must not use this file descriptor
    for anything else---reads and writes should go via the
    @var{ConnectionDriver}.

    @return file descriptor the @var{ConnectionDriver} wants you to
    monitor.

    */
  virtual int getUnderlyingFD()=0;

  /* Provided as a convenience */
  /// Writes the character `c' to the @var{ConnectionDriver}.
  int write(char c) {POFUNC; return write(&c, 1);};
  /// Writes the string `s' to the @var{ConnectionDriver}.
  int write(const char *s) {POFUNC; return write(s, strlen(s));};
  /// Writes the character `c' to the @var{ConnectionDriver}.
  ConnectionDriver &operator<<(char c) {POFUNC; write(&c, 1); return *this;};
  /// Writes the string `s' to the @var{ConnectionDriver}.
  ConnectionDriver &operator<<(const char *s) {POFUNC; write(s, strlen(s)); return *this;};
};

#endif /* CONNECTIONDRIVER_H */
