/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * DisplayDriver.h - Prototypes for generic display driver
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef DISPLAYDRIVER_H
#define DISPLAYDRIVER_H

#include "CharMap.h"

#include <String.h>

class DisplayDriver;
#include "TerminalDriver.h"
#include "ConnectionDriver.h"

struct DisplayDrivers {
  char *name;
  DisplayDriver *(*newobject)(const String &);
};

extern DisplayDrivers DisplayDrivers[];
/**

  @var{DisplayDriver}s are the superclass of classes that provide display
  functions for character mapped displays. Most of the functionality is
  provided by the subclasses which handle, for example Xlib or svgalib
  displays.

  */
class DisplayDriver {
  ///
  void clear(void);
  ///
  void free(void);
  ///
protected:
  ///
  int maxwidth;
  ///
  int maxheight;
  ///
  int truewidth;
  ///
  int trueheight;

  /**
    
    This is a general ticker function that is used by the xlib and svgalib
    display drivers. It performs an update of the flash and cursor blink
    cycles on the @var{TerminalDriver}, updates the display, and works out
    when the next cycle is due.

    @param when On entry, time last tick happened. On exit, time next tick
    is due.
    @param tty TerminalDriver to cycle.

    */
  void TickAction(struct timeval *when, TerminalDriver &tty);
  /// Variable used by TickAction.
  int tickseq;

  /// Callback to tell display to draw itself
  virtual void Draw(TerminalDriver &, BOOL refresh)=0;

  // TerminalDriver used by this display.
  // TerminalDriver *ttydrv;
  ///
public:
  /// Default constructor.
  DisplayDriver(void);
  /// Destructor.
  virtual ~DisplayDriver();

  /**

    This creates a new object of the type requested, which is implemented
    by a subclass of type @var{DisplayDriver}. The display will have
    already been initialised before return. This method will fail if the
    @var{type} is unknown, or the display cannot be opened. Currently there
    is no indication of why it failed---this is a bug.

    @param dn Type of @var{DisplayDriver} to open (@code{"xlib"})
    @param da Arguments for display device. (@code{"800:600:24"})
    @return Pointer to a new object of the type requested, or NULL on failure.

 */
  friend DisplayDriver *newDisplayDriver(const String &dn, const String &da);

  /// @return the physical maximum width of the display.
  int getMaxWidth(void) const {return maxwidth;};
  /// @return the physical maximum height of the display.
  int getMaxHeight(void) const {return maxheight;};
  /// @return the current width used on the display.
  int getTrueWidth(void) const {return truewidth;};
  /// @return the current height used on the display.
  int getTrueHeight(void) const {return trueheight;};

  /**
    +++ FIXME +++
    Main loop for terminal.
    @param sd ConnectionDriver to be used.
    @param td TerminalDriver to be used.
    */
  virtual void Main(ConnectionDriver &sd, TerminalDriver &td)=0;
};

#endif /* DISPLAYDRIVER_H */
