/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * RawFont.cc: Functions to read and process font data.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>

#include <fstream.h>
#include <iostream.h>

#include "../config.h"
#include "global.h"
#include "RawFont.h"

void RawFont::clear(void) {
  POFUNC;
  height=0;
  width=0;
  for(int n=0;n<MAX_FONT_GLYPHS;n++)
    glyph[n]=NULL;
}

void RawFont::free(void) {
  POFUNC;
  for(int n=0;n<MAX_FONT_GLYPHS;n++)
    delete glyph[n];
  clear();
}

RawFont &RawFont::operator=(const RawFont &f) {
  POFUNC;

  free();

  if(!&f) { // Assignment from NULL...
    return *this;
  }

  height=f.height;
  width=f.width;
  name=f.name+"-copy";
  for(int n=0;n<MAX_FONT_GLYPHS;n++) {
    if(f.glyph[n]) {
      if(!(glyph[n]=new BIT[width*height]))
	MEMORY_DIE;
      memcpy(glyph[n], f.glyph[n], width*height*sizeof(BIT));
    }
  }
  return *this;
}


/* FIXME -- requires error checking code */
int RawFont::SaveBin(const String &name, int first, int last) const {
  POFUNC;

  ofstream s;
  s.open(name);

  // Did the open work?
  if(!s) {
    D("Can't save font for some reason");
    return TRUE;
  }

  s.put('R'); s.put('F'); s.put('N'); s.put('T');
  s.put(width);
  s.put(height);
  for(int c=first;c<last;c++) {
    if(glyph[c]) {
      s.put(c);
      int byte=0, bit=0;
      for(int p=0;p<width*height;p++) {
	byte<<=1;
	byte|=(glyph[c][p])?1:0;
	bit++;
	if(bit==8) {
	  s.put(byte);
	  bit=0; byte=0;
	}
      }
      if(bit) {
	byte<<=(8-bit);
	s.put(byte);
      }
    }
  }
  if(!s) { // Oh, how cute, an error occurred.
    D("I/O error writing font");
    return TRUE;
  }
  D("Font saved OK");
  return FALSE;
}

BOOL RawFont::LoadBin(const String &name) {
  POFUNC;

  free();
  D("Attempting to load font " << name);
  ifstream s;
  s.open(name, ios::in|ios::bin);

  // Did we open the file?
  if(!s) {
    D("Font open failed");
    return TRUE;
  }
  
  // Does it have magic?
  if((s.get()!='R')||(s.get()!='F')||(s.get()!='N')||(s.get()!='T')) {
    D("No magic in font");
    return TRUE;
  }
    
  // Okay, let's load it.
  width=s.get();
  height=s.get();
  while(!s.eof()) {
    int cnum;
    if((cnum=s.get())==EOF)
      break;
    delete glyph[cnum];
    glyph[cnum]=new BIT[width*height];
    int byte=0, bit=8;
    for(int p=0;p<width*height;p++) {
      if(bit==8) {
	  byte=s.get();
	  bit=0;
      }
      glyph[cnum][p]=(byte&128)?(~0):0;
      byte<<=1;
      bit++;
    }
    if(!s) {// Any sort of error happened?
      D("I/O error reading font");
      return TRUE;
    }
  }
  // Loaded OK
  D("Font loaded OK");
  return FALSE;
}

static void orarray(BIT *dest, BIT *src, int len) {
  for(;len>0;len--)
    *dest++|=*src++;
}

BOOL RawFont::MakeViewdataGlyphs(void) {
  int gsize=width*height;
  D("Building glyphs, " << width << "x" << height);

  // Fill glyphs 128...191 with block chars
  for(int n=128;n<192;n++) {
    if(!(glyph[n]=new BIT[gsize]))
      return TRUE;
    // Apparently, 'new BIT[]' doesn't clear the memory, so I do it myself
    for(int i=0;i<gsize;i++)
      glyph[n][i]=0;
    if(n&1) orarray(glyph[n],glyph[1], gsize);
    if(n&2) orarray(glyph[n],glyph[2], gsize);
    if(n&4) orarray(glyph[n],glyph[3], gsize);
    if(n&8) orarray(glyph[n],glyph[4], gsize);
    if(n&16) orarray(glyph[n],glyph[5], gsize);
    if(n&32) orarray(glyph[n],glyph[6], gsize);
  }

  // fill glyphs 192...255 with separate block chars
  for(int n=192;n<256;n++) {
    if(!(glyph[n]=new BIT[gsize]))
      return TRUE;
    for(int i=0;i<gsize;i++)
      glyph[n][i]=0;
    if(n&1) orarray(glyph[n],glyph[7], gsize);
    if(n&2) orarray(glyph[n],glyph[8], gsize);
    if(n&4) orarray(glyph[n],glyph[9], gsize);
    if(n&8) orarray(glyph[n],glyph[10], gsize);
    if(n&16) orarray(glyph[n],glyph[11], gsize);
    if(n&32) orarray(glyph[n],glyph[12], gsize);
  }

  // Make double-height characters
  for(int n=0;n<128;n++) {
    if(glyph[n]) {
      if(!(glyph[n+256]=new BIT[gsize]))
	return TRUE;
      if(!(glyph[n+384]=new BIT[gsize]))
	return TRUE;
      int p=0, Y, X;
      for(Y=0;Y<height;Y++) {
	for(X=0;X<width;X++) {
	  glyph[n+256][p]=glyph[n][(Y/2)*width+X];
	  glyph[n+384][p]=glyph[n][((Y+height)/2)*width+X];
	  p++;
	}
      }
    }
  }

  return FALSE;
}
