/*
 * This file is part of XViewdata - a viewdata terminal emulator.
 * Copyright (C) 1997 Peter Corlett. All Rights Reserved.
 *
 * TerminalDriver.h - prototypes for generic terminal driver class
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef TERMINALDRIVER_H
#define TERMINALDRIVER_H

#include "CharMap.h"

#include <String.h>

class TerminalDriver;

struct TerminalDrivers {
  char *name;
  TerminalDriver *(*newobject)(const String &);
};

extern TerminalDrivers TerminalDrivers[];

/// Generic terminal emulation driver.
class TerminalDriver {
  ///
  void clear(void);
  ///
  void free(void);
  ///
protected:
  /// Character map of the display.
  CharMap cmap;
  ///
public:
  /// Default constructor.
  TerminalDriver(void);
  /// Destructor.
  virtual ~TerminalDriver();

  /** Creates a new TerminalDriver object with the appropriate attributes.
    @param dn name of terminal device.
    @param da arguments for terminal device.
    @return NULL on failure, pointer to new TerminalDriver object otherwise.
    */
  friend TerminalDriver *newTerminalDriver(const String &dn, const String &da);

  /** Writes data to the terminal.
    @param buf pointer to the buffer to write.
    @param len length of buffer.
    @return FALSE on success.
    */
  virtual int write(const char *buf, int len)=0;

  /** Reads data from the terminal.
    @param k X keysym value.
    @return string that keysym translates to, or NULL.
    */
  virtual int read(char *buf, int len, int key)=0;

  /** Notify X keyup event.
    @param k X keysym value.
    @return string that keysym translates to, or NULL.
    */
  virtual int keyupread(char *buf, int len, int key)=0;

  /* These are provided as a convenience */
  /// Writes the character `c' to the terminal.
  int write(char c) {POFUNC; return write(&c, 1);};
  /// Writes the string `s' to the terminal.
  int write(const char *s) {POFUNC; return write(s, strlen(s));};
  /// Writes the character `c' to the terminal.
  TerminalDriver &operator<<(char c) {POFUNC; write(&c, 1); return *this;};
  /// Writes the string `s' to the terminal.
  TerminalDriver &operator<<(const char *s) {POFUNC; write(s, strlen(s)); return *this;};

  /// Returns a pointer to the @var{CharMap} used by the terminal.
  CharMap &getCharMap(void) {return cmap;};
  /** Resizes the terminal.
    @param x New width of terminal.
    @param y New height of terminal.
    @return FALSE on failure, in which case the TerminalDriver is invalid.
    */
  virtual BOOL setSize(int x, int y)=0;

  /// Returns the width of the embedded character map.
  int getWidth(void) {return cmap.getWidth();};
  /// Returns the height of the embedded character map.
  int getHeight(void) {return cmap.getHeight();};
};

#endif /* TERMINALDRIVER_H */
